==> jdreact-core-web-0.0.2.tgz: package/local-cli/server.js <==
/**
 * Copyright (c) 2015-present, JingDong Group Holding Limited.
 * All rights reserved.
 */
'use strict';

var chalk = require('chalk');
var path = require('path');
var fs = require('fs');
var easyfile = require('easyfile');
var webpack = require('webpack');
var WebpackDevServer = require('webpack-dev-server');
var historyApiFallback = require('connect-history-api-fallback');
var httpProxyMiddleware = require('http-proxy-middleware');
var clearConsole = require('react-dev-utils/clearConsole');
var formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');
var config = require('./util/config');
var isInteractive = process.stdout.isTTY;
var getRoot = require('./util/config').getRoot;
function setupCompiler(webpackConfig, cliConfig) {
  // "Compiler" is a low-level interface to Webpack.
  // It lets us listen to some events and provide our own custom messages.
  var compiler = webpack(webpackConfig);

  // "invalid" event fires when you have changed a file, and Webpack is
  // recompiling a bundle. WebpackDevServer takes care to pause serving the
  // bundle, so if you refresh, it'll wait instead of serving the old one.
  // "invalid" is short for "bundle invalidated", it doesn't imply any errors.
  compiler.plugin('invalid', function() {
    if (isInteractive) {
      clearConsole();
    }
    console.log('Compiling...');
  });

  var isFirstCompile = true;

  // "done" event fires when Webpack has finished recompiling the bundle.
  // Whether or not you have warnings or errors, you will get this event.
  compiler.plugin('done', function(stats) {
    if (isInteractive) {
      clearConsole();
    }

    // We have switched off the default Webpack output in WebpackDevServer
    // options so we are going to "massage" the warnings and errors and present
    // them in a readable focused way.
    var messages = formatWebpackMessages(stats.toJson({}, true));
    var isSuccessful = !messages.errors.length && !messages.warnings.length;
    var showInstructions = isSuccessful && (isInteractive || isFirstCompile);

    if (isSuccessful) {
      console.log(chalk.green('Compiled successfully!'));
    }

    if (showInstructions) {
      console.log();
      console.log('The app is running at:');
      console.log();
      console.log('  ' + chalk.cyan('http://localhost:' + cliConfig.port + '/'));
      console.log();
      console.log('Note that the development build is not optimized.');
      console.log();
      isFirstCompile = false;
    }

    // If errors exist, only show errors.
    if (messages.errors.length) {
      console.log(chalk.red('Failed to compile.'));
      console.log();
      messages.errors.forEach(message => {
        console.log(message);
        console.log();
      });
      return;
    }

    // Show warnings if no errors were found.
    if (messages.warnings.length) {
      console.log(chalk.yellow('Compiled with warnings.'));
      console.log();
      messages.warnings.forEach(message => {
        console.log(message);
        console.log();
      });
      // Teach some ESLint tricks.
      console.log('You may use special comments to disable some warnings.');
      console.log('Use ' + chalk.yellow('// eslint-disable-next-line') + ' to ignore the next line.');
      console.log('Use ' + chalk.yellow('/* eslint-disable */') + ' to ignore all warnings in a file.');
    }
  });
  return compiler;
}

// We need to provide a custom onError function for httpProxyMiddleware.
// It allows us to log custom error messages on the console.
function onProxyError(proxy) {
  return function(err, req, res){
    var host = req.headers && req.headers.host;
    console.log(
      chalk.red('Proxy error:') + ' Could not proxy request ' + chalk.cyan(req.url) +
      ' from ' + chalk.cyan(host) + ' to ' + chalk.cyan(proxy) + '.'
    );
    console.log(
      'See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
      chalk.cyan(err.code) + ').'
    );
    console.log();

    // And immediately send the proper error response to the client.
    // Otherwise, the request will eventually timeout with ERR_EMPTY_RESPONSE on the client side.
    if (res.writeHead && !res.headersSent) {
        res.writeHead(500);
    }
    res.end('Proxy error: Could not proxy request ' + req.url + ' from ' +
      host + ' to ' + proxy + ' (' + err.code + ').'
    );
  }
}

function addMiddleware(devServer, cliConfig) {
  // `proxy` lets you to specify a fallback server during development.
  // Every unrecognized request will be forwarded to it.
  var proxy = cliConfig.proxy;
  devServer.use(historyApiFallback({
    // Paths with dots should still use the history fallback.
    // See https://github.com/facebookincubator/create-react-app/issues/387.
    disableDotRule: true,
    // For single page apps, we generally want to fallback to /index.html.
    // However we also want to respect `proxy` for API calls.
    // So if `proxy` is specified, we need to decide which fallback to use.
    // We use a heuristic: if request `accept`s text/html, we pick /index.html.
    // Modern browsers include text/html into `accept` header when navigating.
    // However API calls like `fetch()` won’t generally accept text/html.
    // If this heuristic doesn’t work well for you, don’t use `proxy`.
    htmlAcceptHeaders: proxy ?
      ['text/html'] :
      ['text/html', '*/*']
  }));
  if (proxy) {
    if (typeof proxy !== 'string') {
      console.log(chalk.red('proxy must be a string.'));
      console.log(chalk.red('Instead, the type of "proxy" was "' + typeof proxy + '".'));
      process.exit(1);
    }

    // Otherwise, if proxy is specified, we will let it handle any request.
    // There are a few exceptions which we won't send to the proxy:
    // - /index.html (served as HTML5 history API fallback)
    // - /*.hot-update.json (WebpackDevServer uses this too for hot reloading)
    // - /sockjs-node/* (WebpackDevServer uses this for hot reloading)
    // Tip: use https://jex.im/regulex/ to visualize the regex
    var mayProxy = /^(?!\/(index\.html$|.*\.hot-update\.json$|sockjs-node\/)).*$/;

    // Pass the scope regex both to Express and to the middleware for proxying
    // of both HTTP and WebSockets to work without false positives.
    var hpm = httpProxyMiddleware(pathname => mayProxy.test(pathname), {
      target: proxy,
      logLevel: 'silent',
      onProxyReq: function(proxyReq, req, res) {
        // Browers may send Origin headers even with same-origin
        // requests. To prevent CORS issues, we have to change
        // the Origin to match the target URL.
        if (proxyReq.getHeader('origin')) {
          proxyReq.setHeader('origin', proxy);
        }
      },
      onError: onProxyError(proxy),
      secure: false,
      changeOrigin: true,
      ws: true
    });
    devServer.use(mayProxy, hpm);

    // Listen for the websocket 'upgrade' event and upgrade the connection.
    // If this is not done, httpProxyMiddleware will not try to upgrade until
    // an initial plain HTTP request is made.
    devServer.listeningApp.on('upgrade', hpm.upgrade);
  }

  // Finally, by now we have certainly resolved the URL.
  // It may be /index.html, so let the dev server try serving it again.
  devServer.use(devServer.middleware);
}

function runDevServer(compiler, webpackConfig, cliConfig) {
  var devServer = new WebpackDevServer(compiler, {
    // Enable gzip compression of generated files.
    compress: false,
    // Silence WebpackDevServer's own logs since they're generally not useful.
    // It will still show compile warnings and errors with this setting.
    clientLogLevel: 'none',
    // Enable hot reloading server. It will provide /sockjs-node/ endpoint
    // for the WebpackDevServer client so it can learn when the files were
    // updated. The WebpackDevServer client is included as an entry point
    // in the Webpack development configuration. Note that only changes
    // to CSS are currently hot reloaded. JS changes will refresh the browser.
    hot: true,
    // It is important to tell WebpackDevServer to use the same "root" path
    // as we specified in the config. In development, we always serve from /.
    publicPath: webpackConfig.output.publicPath,
    // WebpackDevServer is noisy by default so we emit custom message instead
    // by listening to the compiler events with `compiler.plugin` calls above.
    quiet: true,
    // Reportedly, this avoids CPU overload on some systems.
    // https://github.com/facebookincubator/create-react-app/issues/293
    watchOptions: {
      ignored: /\/node_modules\/(?!@jdreact)/
    },
    // Enable HTTPS if the HTTPS environment variable is set to 'true'
    https: !!cliConfig.https,
    host: cliConfig.host || '0.0.0.0'
  });

  // Our custom middleware proxies requests to /index.html or a remote API.
  addMiddleware(devServer, cliConfig);

  // Launch WebpackDevServer.
  devServer.listen(cliConfig.port, (err, result) => {
    if (err) {
      return console.log(err);
    }
    console.log(chalk.cyan('Starting the development server...'));
    console.log();
  });

}
function copyVersion() {
  const rootDir = getRoot();
  if(!easyfile.isFile(path.join(rootDir, 'web/version.json' ))){
    /**
     * 拷贝 version 文件至 web 目录下，并修改 moduleName
     */
    let fileName = process.cwd().split('-').pop()+'.version';
    let moduleName = process.cwd().split('-').pop();
    if(!easyfile.isFile(fileName)){
      var allfiles = fs.readdirSync(process.cwd()+'/jsbundles/');
      allfiles.forEach(function(item, index){
        if(path.extname(item) == '.version'){
          fileName = item;
          moduleName = path.basename(item,".version");
        }
      })
    }
    let sourceFile = path.join(process.cwd()+'/jsbundles/', fileName);
    let destFile = path.join(rootDir, 'web/version.json' );
    easyfile.copy(sourceFile, destFile, {
      force: true,
      backup: true,
    });

    /**
     * 修改moduleName
     */
    fs.readFile(destFile,'utf8',function (err, data) {
      if(err) console.log(err);
      var version = JSON.parse(data);
      version.moduleName = moduleName;
      var t = JSON.stringify(version);
      fs.writeFileSync(destFile,t);
    });
  }
}
function patchReactDom() {
    /**
     * 拷贝 react-dom.development.js 至 node_modules/react-dom/cjs/react-dom.development.js
     */
    const rootDir = getRoot();
    easyfile.copy(rootDir +'/node_modules/@jdreact/jdreact-core-scripts/patch/react-dom/cjs/react-dom.development.js',
        rootDir +'/node_modules/react-dom/cjs/react-dom.development.js', {
            force: true,
            backup: false,
    });
}
/**
 * Starts the React Web Server.
 */
function server(argv) {
  process.env.NODE_ENV = process.env.NODE_ENV || 'development';

  var cliConfig = config.getCliConfig();
  var webpackConfig = config.getWebpackConfig();
  patchReactDom();
  copyVersion();

  var compiler = setupCompiler(webpackConfig, cliConfig);
  runDevServer(compiler, webpackConfig, cliConfig);
}

module.exports = server;

==> jdreact-core-web-0.0.3.tgz: package/local-cli/server.js <==
'use strict';

var chalk = require('chalk');
var path = require('path');
var fs = require('fs');
var easyfile = require('easyfile');
var webpack = require('webpack');
var WebpackDevServer = require('webpack-dev-server');
var historyApiFallback = require('connect-history-api-fallback');
var httpProxyMiddleware = require('http-proxy-middleware');
var clearConsole = require('react-dev-utils/clearConsole');
var formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');
var config = require('./util/config');
var isInteractive = process.stdout.isTTY;
var getRoot = require('./util/config').getRoot;
function setupCompiler(webpackConfig, cliConfig) {
  // "Compiler" is a low-level interface to Webpack.
  // It lets us listen to some events and provide our own custom messages.
  var compiler = webpack(webpackConfig);

  // "invalid" event fires when you have changed a file, and Webpack is
  // recompiling a bundle. WebpackDevServer takes care to pause serving the
  // bundle, so if you refresh, it'll wait instead of serving the old one.
  // "invalid" is short for "bundle invalidated", it doesn't imply any errors.
  compiler.plugin('invalid', function() {
    if (isInteractive) {
      clearConsole();
    }
    console.log('Compiling...');
  });

  var isFirstCompile = true;

  // "done" event fires when Webpack has finished recompiling the bundle.
  // Whether or not you have warnings or errors, you will get this event.
  compiler.plugin('done', function(stats) {
    if (isInteractive) {
      clearConsole();
    }

    // We have switched off the default Webpack output in WebpackDevServer
    // options so we are going to "massage" the warnings and errors and present
    // them in a readable focused way.
    var messages = formatWebpackMessages(stats.toJson({}, true));
    var isSuccessful = !messages.errors.length && !messages.warnings.length;
    var showInstructions = isSuccessful && (isInteractive || isFirstCompile);

    if (isSuccessful) {
      console.log(chalk.green('Compiled successfully!'));
    }

    if (showInstructions) {
      console.log();
      console.log('The app is running at:');
      console.log();
      console.log('  ' + chalk.cyan('http://localhost:' + cliConfig.port + '/'));
      console.log();
      console.log('Note that the development build is not optimized.');
      console.log();
      isFirstCompile = false;
    }

    // If errors exist, only show errors.
    if (messages.errors.length) {
      console.log(chalk.red('Failed to compile.'));
      console.log();
      messages.errors.forEach(message => {
        console.log(message);
        console.log();
      });
      return;
    }

    // Show warnings if no errors were found.
    if (messages.warnings.length) {
      console.log(chalk.yellow('Compiled with warnings.'));
      console.log();
      messages.warnings.forEach(message => {
        console.log(message);
        console.log();
      });
      // Teach some ESLint tricks.
      console.log('You may use special comments to disable some warnings.');
      console.log('Use ' + chalk.yellow('// eslint-disable-next-line') + ' to ignore the next line.');
      console.log('Use ' + chalk.yellow('/* eslint-disable */') + ' to ignore all warnings in a file.');
    }
  });
  return compiler;
}

// We need to provide a custom onError function for httpProxyMiddleware.
// It allows us to log custom error messages on the console.
function onProxyError(proxy) {
  return function(err, req, res){
    var host = req.headers && req.headers.host;
    console.log(
      chalk.red('Proxy error:') + ' Could not proxy request ' + chalk.cyan(req.url) +
      ' from ' + chalk.cyan(host) + ' to ' + chalk.cyan(proxy) + '.'
    );
    console.log(
      'See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
      chalk.cyan(err.code) + ').'
    );
    console.log();

    // And immediately send the proper error response to the client.
    // Otherwise, the request will eventually timeout with ERR_EMPTY_RESPONSE on the client side.
    if (res.writeHead && !res.headersSent) {
        res.writeHead(500);
    }
    res.end('Proxy error: Could not proxy request ' + req.url + ' from ' +
      host + ' to ' + proxy + ' (' + err.code + ').'
    );
  }
}

function addMiddleware(devServer, cliConfig) {
  // `proxy` lets you to specify a fallback server during development.
  // Every unrecognized request will be forwarded to it.
  var proxy = cliConfig.proxy;
  devServer.use(historyApiFallback({
    // Paths with dots should still use the history fallback.
    // See https://github.com/facebookincubator/create-react-app/issues/387.
    disableDotRule: true,
    // For single page apps, we generally want to fallback to /index.html.
    // However we also want to respect `proxy` for API calls.
    // So if `proxy` is specified, we need to decide which fallback to use.
    // We use a heuristic: if request `accept`s text/html, we pick /index.html.
    // Modern browsers include text/html into `accept` header when navigating.
    // However API calls like `fetch()` won’t generally accept text/html.
    // If this heuristic doesn’t work well for you, don’t use `proxy`.
    htmlAcceptHeaders: proxy ?
      ['text/html'] :
      ['text/html', '*/*']
  }));
  if (proxy) {
    if (typeof proxy !== 'string') {
      console.log(chalk.red('proxy must be a string.'));
      console.log(chalk.red('Instead, the type of "proxy" was "' + typeof proxy + '".'));
      process.exit(1);
    }

    // Otherwise, if proxy is specified, we will let it handle any request.
    // There are a few exceptions which we won't send to the proxy:
    // - /index.html (served as HTML5 history API fallback)
    // - /*.hot-update.json (WebpackDevServer uses this too for hot reloading)
    // - /sockjs-node/* (WebpackDevServer uses this for hot reloading)
    // Tip: use https://jex.im/regulex/ to visualize the regex
    var mayProxy = /^(?!\/(index\.html$|.*\.hot-update\.json$|sockjs-node\/)).*$/;

    // Pass the scope regex both to Express and to the middleware for proxying
    // of both HTTP and WebSockets to work without false positives.
    var hpm = httpProxyMiddleware(pathname => mayProxy.test(pathname), {
      target: proxy,
      logLevel: 'silent',
      onProxyReq: function(proxyReq, req, res) {
        // Browers may send Origin headers even with same-origin
        // requests. To prevent CORS issues, we have to change
        // the Origin to match the target URL.
        if (proxyReq.getHeader('origin')) {
          proxyReq.setHeader('origin', proxy);
        }
      },
      onError: onProxyError(proxy),
      secure: false,
      changeOrigin: true,
      ws: true
    });
    devServer.use(mayProxy, hpm);

    // Listen for the websocket 'upgrade' event and upgrade the connection.
    // If this is not done, httpProxyMiddleware will not try to upgrade until
    // an initial plain HTTP request is made.
    devServer.listeningApp.on('upgrade', hpm.upgrade);
  }

  // Finally, by now we have certainly resolved the URL.
  // It may be /index.html, so let the dev server try serving it again.
  devServer.use(devServer.middleware);
}

function runDevServer(compiler, webpackConfig, cliConfig) {
  var devServer = new WebpackDevServer(compiler, {
    // Enable gzip compression of generated files.
    compress: false,
    // Silence WebpackDevServer's own logs since they're generally not useful.
    // It will still show compile warnings and errors with this setting.
    clientLogLevel: 'none',
    // Enable hot reloading server. It will provide /sockjs-node/ endpoint
    // for the WebpackDevServer client so it can learn when the files were
    // updated. The WebpackDevServer client is included as an entry point
    // in the Webpack development configuration. Note that only changes
    // to CSS are currently hot reloaded. JS changes will refresh the browser.
    hot: true,
    // It is important to tell WebpackDevServer to use the same "root" path
    // as we specified in the config. In development, we always serve from /.
    publicPath: webpackConfig.output.publicPath,
    // WebpackDevServer is noisy by default so we emit custom message instead
    // by listening to the compiler events with `compiler.plugin` calls above.
    quiet: true,
    // Reportedly, this avoids CPU overload on some systems.
    // https://github.com/facebookincubator/create-react-app/issues/293
    watchOptions: {
      ignored: /\/node_modules\/(?!@areslabs)/
    },
    // Enable HTTPS if the HTTPS environment variable is set to 'true'
    https: !!cliConfig.https,
    host: cliConfig.host || '0.0.0.0'
  });

  // Our custom middleware proxies requests to /index.html or a remote API.
  addMiddleware(devServer, cliConfig);

  // Launch WebpackDevServer.
  devServer.listen(cliConfig.port, (err, result) => {
    if (err) {
      return console.log(err);
    }
    console.log(chalk.cyan('Starting the development server...'));
    console.log();
  });

}
function copyVersion() {
  const rootDir = getRoot();
  if(!easyfile.isFile(path.join(rootDir, 'web/version.json' ))){
    /**
     * 拷贝 version 文件至 web 目录下，并修改 moduleName
     */
    let fileName = process.cwd().split('-').pop()+'.version';
    let moduleName = process.cwd().split('-').pop();
    if(!easyfile.isFile(fileName)){
      var allfiles = fs.readdirSync(process.cwd()+'/jsbundles/');
      allfiles.forEach(function(item, index){
        if(path.extname(item) == '.version'){
          fileName = item;
          moduleName = path.basename(item,".version");
        }
      })
    }
    let sourceFile = path.join(process.cwd()+'/jsbundles/', fileName);
    let destFile = path.join(rootDir, 'web/version.json' );
    easyfile.copy(sourceFile, destFile, {
      force: true,
      backup: true,
    });

    /**
     * 修改moduleName
     */
    fs.readFile(destFile,'utf8',function (err, data) {
      if(err) console.log(err);
      var version = JSON.parse(data);
      version.moduleName = moduleName;
      var t = JSON.stringify(version);
      fs.writeFileSync(destFile,t);
    });
  }
}
function patchReactDom() {
    /**
     * 拷贝 react-dom.development.js 至 node_modules/react-dom/cjs/react-dom.development.js
     */
    const rootDir = getRoot();
    easyfile.copy(rootDir +'/node_modules/@areslabs/jdreact-core-scripts/patch/react-dom/cjs/react-dom.development.js',
        rootDir +'/node_modules/react-dom/cjs/react-dom.development.js', {
            force: true,
            backup: false,
    });
}
/**
 * Starts the React Web Server.
 */
function server(argv) {
  process.env.NODE_ENV = process.env.NODE_ENV || 'development';

  var cliConfig = config.getCliConfig();
  var webpackConfig = config.getWebpackConfig();
  patchReactDom();
  copyVersion();

  var compiler = setupCompiler(webpackConfig, cliConfig);
  runDevServer(compiler, webpackConfig, cliConfig);
}

module.exports = server;
